
/*
**
**
** MODULE: client.c
**
**
** PROGRAM: Portable PHNBK2 Application (OpenVMS,DOS,NT,OSF/1,ULTRIX)
**
**
** ABSTRACT: PHNBK2 is a sample RPC application intended to illustrate
**           the basics of RPC interoperation between DCE platforms
**           and Microsoft platforms.  PHNBK2 extends the capabilities
**           of PHNBK by using context handles, and the CDS nameservice.
**
**          
**
**
*/      
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifndef VMS
#include <malloc.h>
#endif
#include "phnbk2.h"

#if defined(MSDOS) || defined(WIN32) 
#include "dceport.h"
#define IFSPEC phnbk2_ClientIfHandle 
#else
#include <pthread_exc.h>
#define IFSPEC phnbk2_v1_0_c_ifspec
#endif

#ifdef WIN32
#define MAIN_DECL _CRTAPI1 
#else
#define MAIN_DECL
#endif

#define SERVER_ENTRY_NAME  "/.:/phnbk2_srv_"
#define SERVER_ENTRY_NAME2 "/.:/subsys/DEC/examples/phnbk2_srv_"

/*
** Internal function prototype
*/
static int find_servers();

int 
MAIN_DECL main 
              (
               ac, 
               av
              )
int   ac;
char *av[];
{

    int               i;                    
    int               lookup_status;       /* lookup return status */
    error_status_t    status;              /* rpc status */
    idl_char          input[LINESIZE];     /* 'find' search string */
    idl_char          output[LINESIZE];    /* string returned */
    idl_char          oldmatch[LINESIZE];  /* previous 'find' string */
    char              command;             /* lookup command */
    unsigned char     server[80];          /* string binding for server */
    phonebook_handle  ph;                  /* phonebook context handle */
  


    /*
    ** Initialize some strings
    */
    oldmatch[0] = '\0';
    server[0]   = '\0';

    /*
    ** Call the find_servers procedure to locate a suitable
    **    phonebook server using the nameservice...
    */
    if (find_servers(server))
       {
       printf("\nServer lookup failed:\n\n\t %s\n",server);
       exit (EXIT_FAILURE);
       }

    /*
    ** Display the server in use
    */
    printf("\n\t(Selected server binding: %s)\n\n", server);       


    /*
    ** Convert the character string binding into an RPC handle
    */
    rpc_binding_from_string_binding
               (
                server, 
                &xhandle, 
                &status
               );

    if (status)
        {
        printf("Invalid string binding\n");
        exit (EXIT_FAILURE);
        }

    /*
    ** Initialize the phonebook context handle to NULL
    */
    ph = NULL;


    /*
    ** Establish the phonebook context handle
    */
    open_phonebook_file
                (
                 &ph
                );


    /* 
    ** Usage -- present user with a list of valid commands
    */ 
    printf("Valid commands are:\n");
    printf("(b)rowse        - List next entry\n");
    printf("(r)eset         - Reset to beginning of file\n");
    printf("(f)ind <string> - Find a substring\n");
    printf("(f)ind          - Find next occurrence of <string>\n");
    printf("(q)uit          - Exit program\n\n");
  
    /*
    ** Keep looking for further commands until the user issues the
    **    (q)uit command...
    */
    while(1)
        {
        /*
        ** Initialize input and output strings
        */
        input[0]  = '\0';
        output[0] = '\0';


        /*
        ** Wait for user input
        */
        command = getchar();   


        /*
        ** Select on user input
        */
        switch(command)
            {
            /*
            ** Command "f" means find the given search 
            **    string, or find the next occurrence 
            **    of the previously supplied search string
            */
            case 'f' :
                
                i = 0;
                /*
                ** skip spaces
                */
                while ((command = getchar()) == ' ');
                                 
                /*
                ** Retrieve search string (if there is one)
                */  
                input[i++] = command;
                    
                if (command != '\n')
                    {
                    /*
                    ** There is a search string so retrieve it
                    */
                    while ((input[i++] = getchar()) != '\n');

                    input[--i] = '\0';    

                    strcpy ( (char *)oldmatch, (char *)input );

                    /*
                    ** Try to find a match for the search string
                    **    in the phonebook file
                    */
                    lookup_status = lookup 
                                          (
                                           ph,
                                           FIRSTMATCH,
                                           input,
                                           output
                                          );
                    /*
                    ** Print the search results: either a
                    **    match was found or it wasn't...
                    */
                    if (lookup_status == END)
                        printf("No matches found\n");
                    else
                        printf("Entry is: %s\n",output);
                    }

                /*
                ** No search string: search for next occurrence 
                **    of the search string given previously
                */
                else
                    {
                    input[0] = '\0';

                    /*
                    ** restore search string
                    */
                    strcpy ((char *)input, (char *)oldmatch);

                    /*
                    ** search for match
                    */
                    lookup_status = lookup
                                          (
                                           ph,
                                           NEXTMATCH,
                                           input,
                                           output
                                          );
                    /*
                    ** Print search results
                    */
                    if (lookup_status == END)
                        printf("No further matches found\n");
                    else
                        printf("Entry is: %s\n",output);
                    }
       
                break;
            /*
            ** Command "b" means just return the next entry
            **    in the phonebook file.
            */
            case 'b' :
     
                /*
                ** Display next entry from wherever we happen to be 
                **    positioned in the phonebook
                */
                lookup_status = lookup
                                      (
                                       ph,
                                       BROWSE, 
                                       input, 
                                       output
                                      );

                /*
                ** Echo entry or rewind database if we're at the end
                **    of the phonebook
                */
                if (lookup_status != END)
                    printf("Entry is: %s\n", output);
                else
                    printf("Resetting...\n");

                break;

            /*
            ** Command "r" means reset file position to
            **    the beginning of the phonebook file
            */   
            case 'r' :
     
                input[0] = '\0';

                /*
                ** Reset phonebook position to beginning 
                */
                lookup_status = lookup
                                      (
                                       ph,
                                       RESET,
                                       input,
                                       output
                                      );

                printf("Resetting...\n");

                break;
           
            /*
            ** Command "q" means exit the program
            */
            case 'q' :
     
                /*
                ** Clean up phonebook context
                */
                close_phonebook_file 
                           (
                            &ph
                           );
                /*
                ** Free binding handle
                */
                rpc_binding_free
                          (
                           &xhandle,
                           &status
                          );


                /*
                ** Exit program
                */
                printf("Exiting...\n");

                exit(EXIT_SUCCESS);

            default :

                break;

            /*
            ** End of "switch" block
            */  
            }
        
        /*
        ** End of "while" block
        */     
        } 
  

}


#if defined(MSDOS) || defined(WIN32)

/*************************************************************************/
/***                MIDL_user_allocate / MIDL_user_free                ***/
/*************************************************************************/

void * __RPC_API
MIDL_user_allocate
           (
            size
           )
size_t size;
{

    unsigned char * ptr;
    ptr = malloc( size );
    return ( (void *)ptr );
    
}

void __RPC_API
MIDL_user_free
           (
            object
           )
void * object;
{
    free (object);
}

#endif
/*
** ROUTINE NAME: find_servers
**
** SCOPE:       PUBLIC
**
** DESCRIPTION
**
**      The purpose of this routine is to retrieve a phnbk2
**      server from the default CDS server and return it as
**      a parameter.
**
** INPUTS: 
**
**      none
**
** OUTPUTS:
**      
**      server  character string
**
** FUNCTION VALUE
**
**      0 success
**      1 failure
**
*/
static int 
find_servers
            (
             server
            )
unsigned char server[80];
{
    unsigned char    *string_binding;
    unsigned char    *protseq;
    handle_t          handle;
    long              entry_name_syntax=3;
    rpc_ns_handle_t   import_context;
    int               found=0;
    error_status_t    status;
    error_status_t    status2;


 TRY

    /*
    ** Get ready to import bindings that match the
    **    given entry name
    */
    rpc_ns_binding_import_begin
               (
                rpc_c_ns_syntax_default,
                (unsigned char *)SERVER_ENTRY_NAME,
                IFSPEC,
                NULL,
                &import_context,
                &status
               );

    if (status)
       {
       strcpy((char *)server, "rpc_ns_binding_import_begin failure");    
       return (1);
       }

    /*
    ** search for TCP binding
    */
    while (!found)
        {
        /*
        ** Get next binding
        */
        rpc_ns_binding_import_next
                   (
                    import_context,
                    &handle,
                    &status
                   );

        if (status)
            { 
            strcpy((char *)server, "server not found");    
            return (1);
            }
        /*
        ** Convert binding to string binding
        */    
        rpc_binding_to_string_binding
                   (
                    handle, 
                    &string_binding,
                    &status
                   );

        if (status)
            {
            strcpy((char *)server, "rpc_binding_to_string_binding failure");    
            return (1);
            }
        else
            {
            /*
            ** Parse the binding into its individual components...
            **    actually all we care about here is the protocol
            **    sequence...
            */
            rpc_string_binding_parse 
                       (
                        string_binding, 
                        NULL, 
                        &protseq, 
                        NULL, 
                        NULL, 
                        NULL,
                        &status
                       );
                 
         
            if (status)
                {
                strcpy((char *)server, "rpc_string_binding_parse failure");    
                return(1);
                }
            else
                {
                /*
                ** Check for a TCP/IP protocol sequence
                */
                if (!(strcmp((char *)protseq, "ncacn_ip_tcp")))
                    /*
                    ** Got one!
                    */ 
                    {
                    strcpy((char *)server, (char *)string_binding);
                    /*
                    ** Free the string binding
                    */                 
                    rpc_string_free
                               (
                                &string_binding, 
                                &status2
                               );
                    found = 1;

                    /*
                    ** finish the search
                    */
                    rpc_ns_binding_import_done(&import_context, &status);
                    }
                else
                    rpc_string_free
                               (
                                &string_binding, 
                                &status2
                               );
                } 

            }
    /*
    ** End of while block
    */
    } 

 CATCH_ALL
#if defined (WIN32) || defined (MSDOS)
    /*
    ** If MSDOS or Windows NT, print the specific error code
    **    and exit
    */
    unsigned long Exception;

    Exception = RpcExceptionCode();

    printf("\n\tException # %d in find_servers...exiting\n",Exception);

#else
    /*
    ** Else, with a "native" DCE platform, let the operating system
    **    take care of printing the exception
    */
    RERAISE;
#endif

    exit (EXIT_FAILURE);

 ENDTRY

    /*
    ** Only way to return successfully
    */   
    return (0);    
}
